# -*- utf-8 -*-
# @Time: 2021/5/30 10:17
# @Author: CACode
import os
import time
from datetime import datetime

from aestate.conf import BASE_ATTR

try:
    import matplotlib.pyplot as plt
except:
    pass


def conversion_types(val):
    """
    将val的类型转换为字符串并插入array
    """
    if isinstance(val, datetime):
        val = val.strftime('%Y-%m-%d %H:%M:%S')
    return val


def date_format(time_obj=time, fmt='%Y-%m-%d %H:%M:%S') -> str:
    """
    时间转字符串
    :param time_obj:
    :param fmt:
    :return:
    """
    _tm = time_obj.time()
    _t = time.localtime(_tm)
    return time.strftime(fmt, _t)


def time_to_datetime(t_time):
    """
    时间戳转datetime
    """
    try:
        d_time = datetime.fromtimestamp(t_time)
    except OSError as ose:
        return None
    return d_time


def get_static_fields(cls):
    """
    获取类的非默认全局变量
    """
    retD = list(set(dir(cls)).difference(set(BASE_ATTR)))
    return retD


def fullname(o):
    """获取对象的类名"""
    module = o.__class__.__module__
    if module is None or module == str.__class__.__module__:
        cls_name = o.__class__.__name__
    else:
        cls_name = module + '.' + o.__class__.__name__

    if cls_name == 'type':
        cls_name = o.__base__.__module__ + '.' + o.__base__.__name__

    return cls_name


def logTupleToText(next_line=True, *content):
    temp = []
    if isinstance(content, str):
        temp.append(content)
    elif isinstance(content, tuple):
        for c in content:
            if isinstance(c, tuple):
                temp.extend(c)
            else:
                temp.append(str(c))
    else:
        temp.append(str(content))
    if next_line:
        temp.append('\n')
    return ''.join([str(_) for _ in temp])


def write(path, *content):
    """
    写出文件
    :param path:位置
    :param content:内容
    :return:
    """
    # 防止有些使用`/`有些用`\\`
    _sep_path = []
    s = path.split('/')
    [_sep_path.extend(item.split('\\')) for item in s]
    _path = ''
    for i in _sep_path:
        _end = _sep_path[len(_sep_path) - 1]
        if i != _end:
            _path += str(i) + os.sep
        else:
            _path += str(i)
        if not os.path.exists(_path):
            if '.' not in i:
                os.makedirs(_path)
    _write_content = logTupleToText(True, *content)
    with open(os.path.join(_path), mode="a", encoding="UTF-8") as f:
        f.write(_write_content)
        f.close()


def get_left_length(node):
    if not node:
        return 0
    if not node.left:
        return 1
    if not node.right:
        return 2 + get_left_length(node.right)
    return 2 + get_left_length(node.left)


def get_right_length(node):
    if not node:
        return 0
    return 1 + get_right_length(node.right)


def get_height(node):
    if not node:
        return 0
    return 1 + max([get_height(node.left), get_height(node.right)])


def get_node_count(node):
    if not node:
        return 0
    return 1 + get_node_count(node.left) + get_node_count(node.right)


def get_fontsize(count):
    if count < 10:
        return 30
    if count < 20:
        return 20
    return 16


def show_node(node, ax, height, index, font_size):
    if not node:
        return
    x1, y1 = None, None
    if node.left:
        x1, y1, index = show_node(node.left, ax, height - 1, index, font_size)
    x = 100 * index - 50
    y = 100 * height - 50
    if x1:
        plt.plot((x1, x), (y1, y), linewidth=2.0, color='b')
    circle_color = "black" if node.is_black_node() else 'r'
    text_color = "beige" if node.is_black_node() else 'black'
    ax.add_artist(plt.Circle((x, y), 50, color=circle_color))
    ax.add_artist(plt.Text(x, y, node.val, color=text_color, fontsize=font_size, horizontalalignment="center",
                           verticalalignment="center"))
    # print(str(node.val), (height, index))

    index += 1
    if node.right:
        x1, y1, index = show_node(node.right, ax, height - 1, index, font_size)
        plt.plot((x1, x), (y1, y), linewidth=2.0, color='b')

    return x, y, index


def draw_node_line(node, ax, height, index):
    x1, y1 = None, None
    if node.left:
        x1, y1, index = draw_node_line(node.left, ax, height - 1, index)
    x = 100 * index - 50
    y = 100 * height - 50
    if x1:
        plt.plot((x1, x), (y1, y), linewidth=2.0, color='b')
    index += 1
    if node.right:
        x1, y1, index = draw_node_line(node.right, ax, height - 1, index)
        plt.plot((x1, x), (y1, y), linewidth=2.0, color='b')

    return x, y, index


def show_rb_tree(tree, title):
    fig, ax = plt.subplots()
    left, right, height = get_left_length(tree), get_right_length(tree), get_height(tree)
    # print(left, right, height)
    plt.ylim(0, height * 100 + 100)
    plt.xlim(0, 100 * get_node_count(tree) + 100)
    show_node(tree, ax, height, 1)
    plt.show()


def save_rb_tree(tree, index):
    fig, ax = plt.subplots()
    fig.set_facecolor('gray')
    left, right, height = get_left_length(tree), get_right_length(tree), get_height(tree)
    # print(left, right, height)
    h = height * 100 + 100
    w = 100 * get_node_count(tree) + 100
    if w < 400:
        w = 400
        h = h * 400 / w
    plt.text(w / 2 - 50, h - 40, index, size=30, family="fantasy", color="r",
             style="italic", weight="light", bbox=dict(facecolor="r", alpha=0.2))
    plt.ylim(0, h)
    plt.xlim(0, w)
    show_node(tree, ax, height, 1, get_fontsize(get_node_count(tree)))

    fig.set_size_inches(10, h / (w / 10))
    plt.savefig("rb/rbtree_{}.png".format(index))


def dp_equals_base(cls, base_cls):
    """
    寻找是否属于cls的基类
    :param cls:
    :param base_cls:
    :return:
    """
    # if cls.__class__
    if cls is None:
        return False
    if cls.__base__ != type:
        if cls.__base__ == base_cls:
            return True
        else:
            return dp_equals_base(cls.__base__, base_cls)
    else:
        return False



from aestate.ajson import aj

from aestate.exception import FieldNotExist
from aestate.util.Log import ALog
from aestate.dbs import _mysql
from aestate.dbs import _mssql
from aestate.work.Adapter import LanguageAdapter

DB_KWARGS = {
    'pymysql': _mysql,
    'pymssql': _mssql
}


class MySqlConfig(_mysql.ParseUtil):
    """
    配置类:
        默认必须携带操作数据库所需的参数:
            - host:数据库地址
            - port:端口
            - database:数据库名
            - user:用户名
            - password:密码
            - charset:编码默认utf8
            - conf:其他配置
    """

    def __init__(self, db_type, *args, **kwargs):
        """
        可用于mysql的示例参数
        :param host:数据库地址
        :param port:端口
        :param database:数据库名
        :param user:用户名
        :param password:密码
        :param charset:编码默认utf8
        :param creator:创建者

        :param db_type:包类型
        """

        if db_type is None:
            ALog.log_error(msg="The creator is missing, do you want to set`db_type=pymysql`?",
                           obj=FieldNotExist, raise_exception=True)
        # 2022/03/02 设置db_type可以为具体的包
        if isinstance(db_type, str):
            self.creator = __import__(db_type)
            opera_name = db_type
        else:
            self.creator = db_type
            opera_name = db_type.__name__
            self.opera = DB_KWARGS[opera_name].OperaBase
        self.opera = DB_KWARGS[opera_name].OperaBase
        self.sqlFields = DB_KWARGS[opera_name].Fields()
        # -----
        self.kw = kwargs
        # 适配器
        if 'adapter' not in kwargs.keys():
            self.adapter = LanguageAdapter()
        super(MySqlConfig, self).__init__()

    def get(self):
        """
        获取当前配置类
        :return:
        """
        return self

    def set_field(self, key, value):
        """
        设置字段
        :param key:键
        :param value:值
        :return:
        """
        setattr(self, key, value)

    def get_field(self, name):
        """
        获取字段
        :param name:
        :return:
        """
        if hasattr(self, name):
            return getattr(self, name)
        return None

    def get_dict(self):
        """
        将配置类转转字典
        :return:
        """
        return self.__dict__

    def get_json(self, bf=False):
        """
        将配置类转json
        :return:
        """
        return aj.parse(self.get_dict(), bf)



import setuptools
from aestate.work.commands import __version__, __description__, __license__, __author_email__, __author__, \
    __project_name__, __url__, __issues__

with open("README.md", "r", encoding="utf-8") as fh:
    long_description = fh.read()

packages = ['aestate', 'aestate.ajson', 'aestate.base', 'aestate.dbs', 'aestate.dbs._mssql', 'aestate.dbs._mysql',
            'aestate.exception', 'aestate.i18n', 'aestate.util', 'aestate.work', 'aestate.work.commands',
            'aestate.work.proxy', 'aestate.work.sql', 'aestate.work.xmlhandler']
print(setuptools.find_packages())

setuptools.setup(
    name=__project_name__,
    # 版本号
    version=__version__,
    # 作者名称
    author=__author__,
    # 作者邮箱
    author_email=__author_email__,
    # 说明文字
    description=__description__,
    # 描述文本
    long_description=long_description,
    # 描述类型
    long_description_content_type="text/markdown",
    # 项目链接
    url=__url__,
    # 项目连接
    project_urls={
        "Bug Tracker": __issues__,
    },
    # 许可证
    license=__license__,
    # 分类
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
        "Topic :: Database",
    ],
    packages=packages,
    python_requires=">=3.7",
    install_requires=['prettytable', 'cryptography', 'psutil', 'DBUtils', 'simplejson'],
    entry_points={
        'console_scripts': [
            'aestate=aestate:start',
        ]
    },
)



class Compulsory(object):
    @staticmethod
    def run_function(func, args, kwargs):
        """
        强制执行
        """
        try:
            return func(*args, **kwargs)
        except TypeError as e:
            pass

        try:
            return func(*args)
        except TypeError as e:
            pass

        try:
            return func(**kwargs)
        except TypeError as e:
            pass

        try:
            return func()
        except TypeError as e:
            pass

        return None

    @staticmethod
    def search_target(module, target_names):

        """
        深度搜素树
        """

        from aestate.util.Log import ALog

        if len(target_names) == 0:
            return module
        # 当前的标记位置
        now_target = target_names[0]
        del target_names[0]
        if hasattr(module, now_target):
            next_module = getattr(module, now_target)
            return Compulsory.search_target(next_module, target_names)
        else:
            ALog.log_error(f'The package name does not exist in the search tree: {now_target}, please check ' +
                           'whether the package name is filled in correctly', ImportError, raise_exception=True)



from aestate.base.model import Model
from aestate.work.Manage import Pojo


def Table(name, msg="", **kwargs):
    """
        标注该类为一个表
        :param name:表的名称
        :param msg:表的描述
        :return:
        """

    def set_to_field(cls: Model):
        dbtype = cls.Config.dbtype(cls.Config.creator)
        pojo = Pojo(config_obj=dbtype, log_conf=cls.Log.__dict__)
        setattr(cls, '__table_name__', name)
        setattr(cls, '__table_msg__', msg)
        cls.instance = pojo
        for key, value in kwargs.items():
            setattr(cls, key, value)
        return cls

    return set_to_field



"""
这个文件用来为pojo对象做标记，当对象为空或为以下任意类型时
insert操做将会忽略该字段，find操作不会处理为空的字段
"""
import datetime
import traceback

from aestate.ajson import aj


class baseTag(object):
    def __init__(self,
                 name=None,
                 length=None,
                 d_point=None,
                 t_type='varchar',
                 is_null=False,
                 primary_key=False,
                 comment="",
                 auto_field=False,
                 auto_time=False,
                 update_auto_time=False,
                 default=None):
        """
        :param name:字段名
        :param length:长度
        :param d_point:小数点
        :param t_type:类型
        :param is_null:允许为空
        :param primary_key:键
        :param comment:注释
        :param auto_field:自增长键
        :param auto_time:默认设置当前时间
        :param update_auto_time:默认设置当前时间并根据当前时间更新
        :param default:默认值
        """
        # 是否为随着时间而更新
        self.update_auto_time = update_auto_time
        if update_auto_time:
            self.default = datetime.datetime.now().strftime('%Y-%m-%d %H:%M:%S')
        # 是否自动设置为当前时间
        self.auto_time = auto_time
        if auto_time:
            self.default = datetime.datetime.now().strftime('%Y-%m-%d %H:%M:%S')
        # 是否为自增
        self.autoField = auto_field
        # 注释
        self.comment = comment
        # 是否为主键
        self.primary_key = primary_key
        # 是否可以为空
        self.is_null = is_null
        # 小数点包含的位数
        self.d_point = d_point
        # 字段的名称
        self.name = name
        # 类型
        self.t_type = t_type
        # 最大长度
        self.length = length
        # 如果有设置自定义默认值则用自定义,如果有其他的条件触发默认值则设置,反之为空
        self.default = default if default else self.default if hasattr(self, 'default') else None
        # 如果使用的是被继承的子类，那么在这里就会有一个名为fields的字段
        # 将所有自定义字段
        if self.fields:
            for key, value in self.fields.items():
                setattr(self, key, value)

            del self.fields

    def get_field(self, name):
        """
        获得字段
        """
        return getattr(self, name)

    def set_field(self, name, value):
        """
        设置值
        """
        setattr(self, name, value)

    def get_table(self, bf):
        """
        获取表数据结构
        """
        if bf:
            return aj.parse(self, bf)
        return aj.load(aj.parse(self))


class Template(baseTag):

    def __init__(self, cls=None, **kwargs):
        self.fields = {}
        if cls:
            kwargs.update(cls.__dict__)
            self.fields['cls'] = cls
        kwargs.update(update_field(**kwargs))
        self.fields.update(kwargs)
        super(Template, self).__init__(**kwargs)


class tinyintField(Template):
    def __init__(self, **kwargs):
        # 是否为booField调用的
        if self.__class__.__name__ == 'boolField' \
                and 'default' in kwargs.keys() \
                and isinstance(kwargs['default'], bool):
            kwargs['default'] = int(kwargs['default'] if 'default' in kwargs.keys() else False)
        super(tinyintField, self).__init__(t_type='tinyint', **kwargs)


class intField(Template):
    def __init__(self, **kwargs):
        super(intField, self).__init__(t_type='int', **kwargs)


class bigintField(Template):
    def __init__(self, **kwargs):
        super(bigintField, self).__init__(t_type='bigint', **kwargs)


class floatField(Template):
    def __init__(self, **kwargs):
        super(floatField, self).__init__(t_type='float', **kwargs)


class doubleField(Template):
    def __init__(self, **kwargs):
        super(doubleField, self).__init__(t_type='double', **kwargs)


class datetimeField(Template):
    def __init__(self, **kwargs):
        super(datetimeField, self).__init__(t_type='datetime', **kwargs)


class charField(Template):
    def __init__(self, **kwargs):
        super(charField, self).__init__(t_type='char', **kwargs)


class varcharField(Template):
    def __init__(self, **kwargs):
        super(varcharField, self).__init__(t_type='varchar', **kwargs)


class textField(Template):
    def __init__(self, **kwargs):
        super(textField, self).__init__(t_type='text', **kwargs)


class tinytextField(Template):
    def __init__(self, **kwargs):
        super(tinytextField, self).__init__(t_type='tinytext', **kwargs)


class longtextField(Template):
    def __init__(self, **kwargs):
        super(longtextField, self).__init__(t_type='longtext', **kwargs)


class boolField(tinyintField):
    """布尔值的字段,必须存在默认值"""

    def __init__(self, **kwargs):
        kwargs.setdefault('default', False)
        super(boolField, self).__init__(**kwargs)


def update_field(**kwargs):
    """
    更新字典配置
    """

    def no_rep(key, value, **kwargs):
        """
        不存在则替换
        """
        if key not in kwargs.keys():
            kwargs[key] = value
        return kwargs

    def has_attr(key, **kwargs):
        if key in kwargs.keys():
            return kwargs[key]
        return None

    # kwargs.update(no_rep('table_name', has_attr('__table_name__', **kwargs), **kwargs))
    kwargs.update(no_rep('name', has_attr('name', **kwargs), **kwargs))
    kwargs.update(no_rep('length', has_attr('length', **kwargs), **kwargs))
    kwargs.update(no_rep('d_point', has_attr('d_point', **kwargs), **kwargs))
    kwargs.update(no_rep('t_type', has_attr('t_type', **kwargs), **kwargs))
    kwargs.update(no_rep('is_null', has_attr('is_null', **kwargs), **kwargs))
    kwargs.update(no_rep('primary_key', has_attr('primary_key', **kwargs), **kwargs))
    kwargs.update(no_rep('comment', has_attr('comment', **kwargs), **kwargs))
    kwargs.update(no_rep('auto_field', has_attr('auto_field', **kwargs), **kwargs))
    kwargs.update(no_rep('auto_time', has_attr('auto_time', **kwargs), **kwargs))
    kwargs.update(no_rep('update_auto_time', has_attr('update_auto_time', **kwargs), **kwargs))
    return kwargs



from typing import Tuple


class OperaBase:
    def __init__(self, instance):
        self.instance = instance
        self.R = None

    def extra(self, field) -> Tuple[bool, object]: ...

    def check(self): ...

    def create(self): ...



import sys

from aestate.work.commands import Commands

DEBUG = False
DEBUG_ARGS = ('aestate', '-create', 'testOpera.table.demoModels', 'demo')


def start():
    avg = sys.argv
    print(avg)
    if DEBUG:
        avg = DEBUG_ARGS
    _ = Commands(*avg)
    if len(avg) == 1:
        _.c[''][0]()
    else:
        _.c[avg[1]][0]()


if __name__ == '__main__':
    start()



import re
from enum import Enum
from typing import List

from aestate.exception import SqlResultError
from aestate.util.Log import ALog
from prettytable import PrettyTable

from aestate.work.Serialize import QuerySet


class BaseCover:

    def table_visual(self, title: list, val: List[dict]) -> PrettyTable:
        if not val:
            ALog.log_error(
                msg='The database gets no return from the SQL of the field type in the table,'
                    ' and the SQL is written incorrectly, please check the SQL',
                obj=SqlResultError, raise_exception=True)

        table = PrettyTable(title)
        table.border = True
        [table.add_row(list(i.values())) for i in val]
        return table

    def check(self, res, fields) -> bool:
        """
        检查表与数据库中的数据是否对应
        """
        comment = {
            'auto_increment': "自增",
            'DEFAULT_GENERATED': "自动追加日期",
            'DEFAULT_GENERATED on update CURRENT_TIMESTAMP': "自动更新时间为最后一次更改时间"
        }
        return False

    def deal(self, res, fields):
        """
        同步数据库与pojo的字段配置项
        """
        return True

    def res(self, tb_name, db_name, extool) -> dict:
        """
        获得表结构
        """
        sql = """SELECT 
        COLUMN_NAME AS name, -- 名称
        DATA_TYPE AS typer, -- 类型
        CHARACTER_MAXIMUM_LENGTH AS length, -- 长度
        NUMERIC_SCALE AS num_scale, -- 数字小数点
        IS_NULLABLE AS is_null, -- 是否允许为空
        COLUMN_KEY AS c_key, -- 是否为键
        EXTRA as def_val, -- 默认值
        COLUMN_COMMENT as comment -- 描述
        FROM information_schema.COLUMNS WHERE TABLE_SCHEMA = %s AND TABLE_NAME = %s"""
        # 使用传入的sql执行器执行查询表结构的sql
        res = extool.select(sql=sql, params=[db_name, tb_name])
        # 获得表结构可视化
        # table = self.table_visual(list(res[0].keys()), res)
        # print(table)

        # 对比结构

        return res


class AOrm(object):
    """
    纯净的ORM模式:
        你可以使用find('table').by('args').order_by('args').desc().end()方式执行sql
        好处就是:
            能更好的拒绝sql注入
            无需写sql语句
            代码简单易懂
            上手快
    """

    class Mode(Enum):
        FIND = 0
        INSERT = 1

    def __init__(self, repository):
        """
        初始化ORM
        自定义方言除了可使用默认已存在的方法，还可以使用自定义sql方言拼接
        :param repository:仓库
s        """
        self.exmode = None
        self.args = []
        self.params = []
        self.sqlFields = None
        # self.sqlFields = sqlFields
        self.ParseUtil = repository.config_obj
        self.serializer = repository.serializer
        self.sqlFields = repository.sqlFields
        # 创建sql语法
        if repository is None:
            ALog.log_error(
                msg='Repository is null,Place use repository of ORM framework',
                obj=AttributeError, LogObject=self.repository.log_obj, raise_exception=True)
        self.repository = repository
        # self.__table_name__ = '{}{}{}'.format(self.sqlFields.left_subscript, repository.__table_name__,
        #                                       self.sqlFields.right_subscript)
        self.__table_name__ = repository.__table_name__

        self.first_data = False
        self.result = []

    def top(self):
        self.find()
        self.first_data = True
        return self.limit(1)

    def first(self):
        """
        是否只返回第一行数据
        """
        self.first_data = True
        return self

    # ------------------------主键--------------------------
    def find(self, *args, **kwargs):
        """
        查
        example:
            find('all')
            find('param1',asses=['p'],h_func=True)
        Attributes:
            asses:将对应的字段转成另一个别名,不需要转换的使用None标识
            h_func:不将字段转换成 `%s` 格式
        更新:
            如果args字段长度为0,默认为查找全部
        """
        self.exmode = AOrm.Mode.FIND
        self.args.append(self.sqlFields.find_str)
        # 如果有as字段
        alias = None
        if 'alias' in kwargs.keys():
            alias = kwargs['alias']
        # 如果包含方法的字段，则不加密
        func_flag = False
        if 'h_func' in kwargs.keys():
            func_flag = kwargs['h_func']
        # 1.1.0.05更新,默认为all
        _all = False
        if len(args) == 0:
            _all = True
        # 如果存在all
        # 1.1.1.2修复：tuple index out of range
        if _all or 'all'.upper() == args[0].upper():
            # 如果包含all关键字,则使用解析工具解析成字段参数
            if not func_flag:
                fields = self.ParseUtil.parse_key(
                    *self.repository.fields, is_field=True)
            else:
                fields = self.ParseUtil.parse_key(
                    *self.repository.fields, is_field=False)
        else:
            if not func_flag:
                fields = self.ParseUtil.parse_key(*args, is_field=True)
            else:
                fields = self.ParseUtil.parse_key(*args, is_field=False)
        # 解决as问题
        if alias is not None:
            fs = fields.split(',')
            if len(fs) != len(alias):
                # 匿名参数长度与字段长度不符合
                ALog.log_error(obj=TypeError,
                               msg='The length of the anonymous parameter does not match the length of the field',
                               raise_exception=True)
            for i, v in enumerate(fs):
                if alias[i] is not None:
                    self.args.append('{}{}{}'.format(
                        v, self.sqlFields.asses_str, alias[i]))
                else:
                    self.args.append(v)
                # 逗号
                self.args.append(self.sqlFields.comma)
        else:
            self.args.append(fields)
        if alias is not None:
            # 去掉末尾的逗号
            self.rep_sym()
        #     加上from关键字
        if 'poly' not in kwargs.keys():
            self.con_from()
        else:
            self.args += kwargs['poly']
        return self

    def order_by(self, *args, **kwargs):
        """
        根据什么查
        example:
            find('all').order_by('param')
            find('all').order_by('param').end()
            find('all').order_by('p1','p2').desc().limit(10,20)
        """
        return self.by_opera(field=self.sqlFields.order_by_str, args_list=args, **kwargs)

    def group_by(self, *args, **kwargs):
        """
        聚合函数
        example:
            select shop_id,count(*) as count from comments group by shop_id having count>1;
        """
        return self.by_opera(field=self.sqlFields.group_by_str, args_list=args, **kwargs)

    def by_opera(self, field, args_list, **kwargs):
        """
        根据什么查
        """
        self.args.append(field)
        for i in args_list:
            if not kwargs.get('text', False):
                self.args.append(self.sqlFields.left_subscript)
            self.args.append(i)
            if not kwargs.get('text', False):
                self.args.append(self.sqlFields.right_subscript)
            self.args.append(self.sqlFields.comma)
        self.rep_sym(self.sqlFields.comma, self.sqlFields.space)
        return self

    def filter(self, **kwargs):
        return self.find().where(**kwargs).end()

    async def filter_async(self, **kwargs):
        return self.filter(**kwargs)

    def where(self, **kwargs):
        """
        当....
        example:
            find('ALL').where(param='%s') - 默认符号为等于 ==
            find('ALL').where(param='==%s')
            find('ALL').where(param='>%d')
            find('ALL').where(param='<%d')
            find('ALL').where(param='<=%d')
            find('ALL').where(param='>=%.2f')
            find('ALL').where(param='!=%.2f')
        复杂语法:
            find('ALL').where(param='+%d/%d==%d')
            find('ALL').where(param='-%.2f*%d==12')
            find('ALL').where(param='*10-1==12')
            find('ALL').where(param='/10+1==12')
        """
        self.args.append(self.sqlFields.where_str)
        for key, value in kwargs.items():
            cp_key = key
            customize = False
            sym = '='

            sps = cp_key.split('__')
            rpx_symbol = re.findall(r"([%s]{2})(.*)" % ','.join(self.sqlFields.symbol), str(value))
            # UPDATE [1.0.6a2] 使用正则方式替换,减少误判情况造成无法使用特殊运算符拼接sql
            # if len(str(value)) > 2 and str(value)[0:2] in self.sqlFields.symbol:
            if len(rpx_symbol) > 0:
                params = rpx_symbol[0]
                sym, value = params[0], params[1]
                if sym == '==':
                    sym = '='
                elif sym == '>>':
                    sym = '>'
                elif sym == '<<':
                    sym = '<'
                else:
                    # 没有找到符号的话就从字段名开始
                    # 截取最后一段从两段下划线开始的末尾
                    if not len(sps) == 1:
                        customize = True
                        sym = sps[len(sps) - 1]
                        self.ParseUtil.fieldExist(
                            self.ParseUtil, 'adapter', raise_exception=True)
                        cp_key = cp_key[:cp_key.rfind('__' + sym)]
                        self.ParseUtil.adapter.funcs[sym](self, cp_key, value)

            elif not len(sps) == 1:
                customize = True
                sym = sps[len(sps) - 1]
                self.ParseUtil.fieldExist(self.ParseUtil, 'adapter', raise_exception=True)
                cp_key = cp_key[:cp_key.rfind('__' + sym)]
                self.ParseUtil.adapter.funcs[sym](self, cp_key, value)

            if not customize:
                self.args.append(
                    '{}{}{}{}%s'.format(self.sqlFields.left_subscript,
                                        cp_key,
                                        self.sqlFields.right_subscript,
                                        sym))
                self.args.append(self.sqlFields.ander_str)
                self.params.append(value)
        self.rep_sym(self.sqlFields.ander_str)

        return self

    def on(self, from_where, to_where, symbol='='):
        self.args.append(self.sqlFields.space)
        self.args.append(self.sqlFields.on_str)
        self.args.append(self.sqlFields.space)
        self.args.append(from_where)
        self.args.append(self.sqlFields.space)
        self.args.append(symbol)
        self.args.append(self.sqlFields.space)
        self.args.append(to_where)
        return self

    def limit(self, start=0, end=None):
        """
        分页
        :param start:开始
        :param end:末尾
        example:
            find('all').limit(start=10,end=20)
            find('all').limit(end=10)
        """
        self.args.append(self.sqlFields.limit_str)
        # 死亡空格
        if end is None:
            limit_param = '{}{}{}'.format(
                self.sqlFields.space, start, self.sqlFields.space)
        else:
            limit_param = '{}{}{}{}{}'.format(self.sqlFields.space, start, self.sqlFields.comma, end,
                                              self.sqlFields.space)
        self.args.append(limit_param)
        return self

    def desc(self):
        """
        倒叙
        example:
            find('all').desc()
            find('all').desc().end()
            find('all').order_by('param').desc().limit(10,20)
        """

        if self.sqlFields.order_by_str not in self.args:
            ALog.log_error(
                'There is no `order by` field before calling `desc` field,You have an error in your SQL syntax',
                AttributeError, LogObject=self.repository.log_obj, raise_exception=True)

        self.args.append(self.sqlFields.desc_str)
        return self

    def set(self, **kwargs):
        """
        设置
        example:
            update('table').set('param','value').end()
            update('table').set('param1','value1').where('param2=value2').end()
        """
        self.args.append(self.sqlFields.set_str)
        _size = len(kwargs.keys())
        for key, value in kwargs.items():
            self.args.append('{}{}{}{}%s'.format(self.sqlFields.left_subscript,
                                                 key,
                                                 self.sqlFields.right_subscript,
                                                 self.sqlFields.eq))
            # set是加逗号不是and
            self.args.append(self.sqlFields.comma)
            self.params.append(value)
        self.rep_sym(self.sqlFields.comma)
        return self

    # ------------------------预设符--------------------------

    def ander(self):
        """
        和
        example:
            update('table').set('param1','value1').and().set('param2','value2')
            update('table').set('param1','value1').and().set('param2','value2').end()
            update('table').set('param1','value1').and().set('param2','value2').where('param3=value3').end()
        """
        self.args.append(self.sqlFields.ander_str)
        return self

    def __run__(self, need_sql=False, serializer=True) -> QuerySet:
        """
        最终执行任务
        """
        sql = ''
        conf = self.ParseUtil.get_dict()
        print_sql = 'print_sql' in conf.keys() and conf['print_sql'] is True
        last_id = 'last_id' in conf.keys() and conf['last_id'] is True
        sql += ''.join(self.args)
        if need_sql:
            return sql
        if self.exmode == AOrm.Mode.FIND:
            self.result = self.repository.db_util.select(
                sql=sql,
                params=self.params,
                print_sql=print_sql,
                last_id=last_id,
                **self.repository.__dict__
            )
            _result_objs = []
            if self.result is not None:
                for i in self.result:
                    _obj = self.ParseUtil.parse_obj(
                        data=i, instance=self.repository.instance)
                    _result_objs.append(_obj)
            self.result = _result_objs
        else:
            self.result = self.repository.db_util.update(
                sql=sql,
                params=self.params,
                print_sql=print_sql,
                last_id=last_id
            )
        # 清空资源，为下一次使用做准备
        self.args.clear()
        self.params.clear()
        if self.first_data:
            if (isinstance(self.result, list) or isinstance(self.result, tuple)) and self.result and len(
                    self.result) > 0:
                if not serializer:
                    return self.result[0]
                self.repository.result = self.serializer(instance=self.repository.instance,
                                                         base_data=self.result).first()
                return self.repository.result
            else:
                self.repository.result = self.serializer()
                return self.repository.result
        else:
            if not serializer:
                return self.result
            self.repository.result = self.serializer(
                instance=self.repository.instance, base_data=self.result)
            return self.repository.result

    def con_from(self):
        """
        如果没有被from包含,则在末尾加上from __table_name__关键字
        """
        if self.sqlFields.from_str not in self.args:
            self.args.append(self.sqlFields.from_str)
            # 然后加上表名
            self.args.append(self.sqlFields.left_subscript)
            self.args.append(self.__table_name__)
            self.args.append(self.sqlFields.right_subscript)

    def append(self, app_sql):
        """
        末尾追加一些sql
        """
        self.args.append(app_sql)
        return self

    def rep_sym(self, sym=',', rep=''):
        """
        将最后一个参数包含的指定字符替换为指定字符
        """
        self.args[len(self.args) -
                  1] = str(self.args[len(self.args) - 1]).replace(sym, rep)
        return self

    def end(self, **kwargs):
        return self.__run__(**kwargs)

    async def end_async(self, **kwargs):
        return self.__run__(**kwargs)

    def __rshift__(self, other):
        """
        将左边orm迁移至右边
        """
        new_args = self.args.copy()
        new_args.append(' ) ')
        other.args.append(' ( ')
        other.args += new_args
        return other

    def __lshift__(self, other):
        """
        将右边迁移至左边
        """
        new_args = other.args.copy()
        new_args.append(' ) ')
        self.args.append(' ( ')
        self.args = self.args + new_args
        return self

    def alias(self, name):
        """
        设置别名
        """
        self.args.append(' AS ')
        self.args.append(name)
        return self

    def left_join(self, sql_orm, name):
        """
        left join
        """
        self.args.append(self.sqlFields.left_join_str)
        self.args.append(sql_orm.__table_name__)
        return self.alias(name)

    def __str__(self):
        sql = ''.join(self.args)
        return sql

    def check(self):
        """sudo apt install containerd
        检查表结构与数据库中是否对应
        """
        return self.repository.config_obj.opera(self.repository).check()

    def create(self, replace=False):
        return self.repository.config_obj.opera(self.repository).create(replace)



# -*- utf-8 -*-
import inspect

from aestate.work.proxy.SqlOperaProxy import RepositoryProxy


class BaseManager:
    @classmethod
    def _get_queryset_methods(cls, queryset_class):
        def create_method(name, method):
            def manager_method(self, *args, **kwargs):
                # obj = object.__new__(queryset_class)
                func = getattr(self, name)
                return func(self, *args, **kwargs)

            manager_method.__name__ = method.__name__
            manager_method.__doc__ = method.__doc__
            return manager_method

        new_methods = {}
        for name, method in inspect.getmembers(queryset_class, predicate=inspect.isfunction):
            # Only copy missing methods.
            if hasattr(cls, name):
                continue
            # Copy the method onto the manager.
            new_methods[name] = create_method(name, method)
        return new_methods

    @classmethod
    def from_queryset(cls, queryset_class, class_name=None):
        if class_name is None:
            class_name = '%sFrom%s' % (cls.__name__, queryset_class.__name__)
        ms = cls._get_queryset_methods(queryset_class)
        return type(class_name, (cls,), {
            'model_class': queryset_class,
            **ms
        })


class ModelBase(BaseManager.from_queryset(RepositoryProxy)):
    def __new__(cls, class_name, class_parent, class_attr):
        return type(class_name, class_parent, class_attr)
